require 'spec_helper'
require 'tmpdir'

describe "packagelcoud debtools parsers", type: :integration do
  describe "parses official ubuntu debs" do
    before(:all) do
      @ubuntu_archive_url = "http://us.archive.ubuntu.com/ubuntu/"
      @cache_dir = Dir.mktmpdir
      @ubuntu_index = get_ubuntu_index("Precise", @ubuntu_archive_url)
      @packages = get_list_of_pkgs(@ubuntu_index, 5242880)
    end

    after(:all) do
      FileUtils.remove_entry @cache_dir
    end

    it "generates strings which match the raw bytes in the index" do
      @packages[0..300].each do |pkg|
        file_basename = ::File.basename(pkg[:filename])
        pkg_url = URI.join(@ubuntu_archive_url, pkg[:filename])
        cached_deb = get_package_file(file_basename, pkg_url, @cache_dir)
        deb = PackagecloudDebtools::Parser.parse_from_file(cached_deb)

        pkg[:stanza].each_line do |line|
          next if line =~ /^[\t\s]+/

          chunks = line.split(':', 2)
          field = chunks[0].strip.downcase

          if is_relationship?(field)
            value = deb.fields['relationships'][field]
          elsif field == 'section' || field == 'priority'
            # these two fields have different values in the index files
            # than are found in the debs, so we can't check these...
            next
          elsif field == 'filename' || field == 'bugs' || field == 'origin' ||
                field == 'task' || field == 'build-essential' || field == 'supported'
            # these fields appear to be added by some post-processing script;
            # in other words, these fields dont exist in the deb, but somehow
            # end up in the ubuntu index files (??)
            next
          elsif field == 'sha1'
            value = deb.fields['sha1sum']
          elsif field == 'sha256'
            value = deb.fields['sha256sum']
          elsif field == 'sha512'
            value = deb.fields['sha512sum']
          elsif field == 'description'
            value = deb.fields['description'].split("\n").first
          elsif field == 'description-md5'
            value = deb.fields['description_md5']
          else
            value = deb.fields[field]
          end

          parser_string = "#{capitalize_deb_field(field)}: #{value}\n"
          expect(parser_string).to eq(line) 
        end
      end
    end

    it "matches dpkg-deb output" do
      @packages[0..300].each do |pkg|
        file_basename = ::File.basename(pkg[:filename])
        pkg_url = URI.join(@ubuntu_archive_url, pkg[:filename])
        cached_deb = get_package_file(file_basename, pkg_url, @cache_dir)
        deb = PackagecloudDebtools::Parser.parse_from_file(cached_deb)

        deb.fields.each do |k, v|
          if k == 'size'
            sz = ::File.stat(cached_deb).size
            expect(v).to eq(sz)
          elsif k == 'files'
            # files arent written out to Package indices, so skip for now
          elsif k == 'relationships'
            v.each do |k, v|
              fieldvalue = dpkg_deb(k, cached_deb)
              expect(v).to eq(fieldvalue)
            end
          elsif k == 'md5sum' || k == 'sha1sum' || k == 'sha256sum' || k == 'sha512sum'
            sum = sum_from_file(k, cached_deb)
            expect(v).to eq(sum)
          else
            fieldvalue = dpkg_deb(k, cached_deb)
            if k == 'description'
              fieldvalue += "\n"
            end

            if k == 'description_md5'
              description = dpkg_deb("description", cached_deb)
              description += "\n"
              md5 = Digest::MD5.hexdigest(description)
              expect(v).to eq(md5)
            else
              expect(v).to eq(fieldvalue)
            end
          end
        end
      end
    end
  end
end
